﻿using RimWorld;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Verse;
using Verse.Sound;

namespace AIRyrtwofAddon.Abnos.WCCA
{
    public class CompAbnos_WeCanChangeAnything: CompAbnos_Base, IThingHolder
    {
        private ThingOwner innerContainer; 
        private SoundDef sound;

        private int asseptDamage = 0;

        public int Power => asseptDamage;

        public override void PostSpawnSetup(bool respawningAfterLoad)
        {
            action = new Command_Target()
            {
                targetingParams = new TargetingParameters()
                {
                    validator = (TargetInfo t) =>
                    {
                        if (t == null || !t.IsValid || !t.HasThing || !(t.Thing is Pawn)) return false;
                        Pawn p = t.Thing as Pawn;
                        if(!p.RaceProps.Humanlike || !(p.Faction.IsPlayer || p.IsPrisonerOfColony )) return false;
                        return true;
                    },
                },
                defaultLabel = parent.Label + "を使用する",
                icon = parent.def.uiIcon,
                /*action = (LocalTargetInfo t) =>
                {
                    if (actor != null) return;
                    Pawn p = t.Thing as Pawn;

                    Verse.AI.Job tmpjob = JobMaker.MakeJob(AIRDefOf.AIRJob_ApproachAbnos, parent);
                    p.jobs.TryTakeOrderedJob(tmpjob);

                },*/
                
                action = (Thing t) => {
                    if (actor != null) return;
                    Pawn p = t as Pawn;

                    Verse.AI.Job tmpjob = JobMaker.MakeJob(AIRDefOf.AIRJob_ApproachAbnos, parent);
                    p.jobs.TryTakeOrderedJob(tmpjob);
                },
            };
        }

        public override void CompTick()
        {
            //if (innerContainer != null && innerContainer.HasData()) innerContainer.ThingOwnerTick();
            if (innerContainer != null && !innerContainer.NullOrEmpty()) innerContainer.ThingOwnerTick();

            int times = TICK_TIME / Math.Min(jobTick / 5 + 1, 10);

            count = count > times ? 0 : count + 1;
            if (count % times != times - 1) return;

            if (finished)
            {
                jobTick = TIME_NOJOB;
                finished = false;
            }
            else if (approached)
            {
                jobTick = TIME_ACTJOB;
                innerContainer = new ThingOwner<Thing>(this);
                innerContainer.TryAddOrTransfer(actor.SplitOff(1));

                approached = false;
            }
            else if (jobTick != TIME_NOJOB)
            {
                jobTick++;
                asseptDamage = jobTick;

                BodyPartRecord body = actor.health.hediffSet
                        .GetRandomNotMissingPart(DamageDefOf.Stab);
                DamageInfo info = new DamageInfo(DamageDefOf.Stab, 1, 5,
                    instigator: parent,
                    hitPart: body);
                actor.TakeDamage(info);

                if (sound == null) sound = parent.def.GetCompProperties<CompProperties_Abnos>().workSound;
                if (sound != null) sound.PlayOneShot(SoundInfo.InMap(new TargetInfo(parent)));

                if (actor.Dead || jobTick >= 10000 )
                {
                    if (jobTick >= 10000)
                    {
                        innerContainer.TryDropAll(parent.Position, actor.Map, ThingPlaceMode.Direct);
                        actor.Kill(null);
                    }

                    innerContainer.Remove(actor);
                    Pawn pawn = actor;
                    finished = true;
                    actor = null;

                    IntVec3 pos = parent.Position + IntVec3.South;
                    for (int i = 0; i < pawn.GetStatValue(StatDefOf.MeatAmount); i++)
                    {
                        GenSpawn.Spawn(pawn.RaceProps.meatDef, pos, parent.Map);
                    }
                    if(pawn.Corpse!=null) pawn.Corpse.Destroy();
                }
            }
            else
            {
                if (asseptDamage > 0) asseptDamage--;
            }
        }

        public override void PostDeSpawn(Map map)
        {
            // v1.3
            //if (innerContainer != null && innerContainer.HasData())
            if (innerContainer != null && !innerContainer.NullOrEmpty())
            {
                innerContainer.TryDropAll(parent.Position,map,ThingPlaceMode.Direct);
                actor.Kill(null);
            }
        }



        public override IEnumerable<Gizmo> CompGetGizmosExtra()
        {
            //foreach (Gizmo g in base.CompGetGizmosExtra()) yield return g;
            yield return action;
        }

        public void GetChildHolders(List<IThingHolder> outChildren)
        {
            ThingOwnerUtility.AppendThingHoldersFromThings(
                outChildren, GetDirectlyHeldThings()
                );
        }

        public ThingOwner GetDirectlyHeldThings() => this.innerContainer;

        public override void PostExposeData()
        {
            base.PostExposeData();

            Scribe_References.Look(ref actor, "actor");
            Scribe_Deep.Look(ref innerContainer, "abnosInnerContainer", new object[] { this });
            Scribe_Values.Look(ref jobTick, "jobTick", TIME_NOJOB);

        }



    }
}
