﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using Verse;
using RimWorld;
using UnityEngine;

namespace AIRyrtwofAddon.Information
{
    [StaticConstructorOnStartup]
    public static class InfoContentCache
    {
        private static InfoWindow window = InfoWindow.Instance;

        private static string staff = "入植者";
        private static string colony = "入植地";

        private static Texture2D cmdOpen = 
            ContentFinder<Texture2D>.Get("Icon/Menu/open");

        public static StringColor[] features = {
            new StringColor(staff + "が即死する恐れあり", "即死", Color.red),
            new StringColor(staff + "が変異する恐れあり","変異"),
            new StringColor(staff + "が支配される恐れあり" , "支配"),
            new StringColor("脱走する恐れあり" , "脱走"),
            new StringColor(colony+ "に恩恵を与える可能性あり", "恩恵" , Color.green),
            new StringColor(colony + "が崩壊する恐れあり","崩壊", Color.red),
        };

        public static Color[] dangerColor = { 
            Color.green,
            Color.blue,
            Color.yellow,
            Color.magenta,
            Color.red,
        };

        public static Color[] FColor =
        {
            Color.yellow,
            Color.green,
            Color.red,
            Color.white,
        };


        public static Command_Action OpenInfo(ThingDef t)
        {
            Command_Action action = new Command_Action();
            action.defaultLabel = "情報";
            action.action = () => window.Open(t);
            action.icon = cmdOpen;
            action.activateSound = SoundDefOf.InfoCard_Open;

            return action;
        }

        public static float Height(this ImageDesc id, float width)
        {
            return Text.CalcHeight(id.desc, width) + 14f + 
                (id.imagePath != null ? width - 20f : 0f);
        }

        private static Dictionary<string, Texture2D> cacheTex = new Dictionary<string, Texture2D>();
        public static Texture2D Get(this ImageDesc id)
        {
            if (cacheTex.ContainsKey(id.imagePath)) 
                return cacheTex[id.imagePath];

            Texture2D tex = ContentFinder<Texture2D>.Get(id.imagePath);
            cacheTex.Add(id.imagePath, tex);

            return tex;
        }

    }
    public class StringColor
    {
        public string desc;
        public Color color = Color.yellow;
        public string tag;

        public StringColor() { }
        
        public StringColor(string desc) : this(desc,Color.yellow){}
        public StringColor(string desc, Color color) : this(desc, null, color) { }

        public StringColor(string desc, string tag) : this(desc, tag, Color.yellow) { }

        public StringColor(string desc, string tag, Color color)
        {
            this.desc = desc;
            this.color = color;
            this.tag = tag;
        }

        public override string ToString() => "[" +desc + color +"]";
    }

    public struct ImageDesc
    {
        public string desc;
        public string imagePath;
    }



}
