﻿using AIRyrtwofAddon.Information;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using UnityEngine;
using Verse;

namespace AIRyrtwofAddon
{
    [StaticConstructorOnStartup]

    public static class WindowTool
    {
        public static readonly Texture2D DeleteX = ContentFinder<Texture2D>.Get("UI/Buttons/Delete", true);
        public static readonly Texture2D Plus = ContentFinder<Texture2D>.Get("UI/Buttons/Plus", true);
        public static readonly Texture2D Minus = ContentFinder<Texture2D>.Get("UI/Buttons/Minus", true);

        private static Color gray = new ColorInt(50, 52,54).ToColor;

        private static Color menuColor = new ColorInt(42, 43, 44).ToColor;
        private static Color menuBorderColor = new ColorInt(135, 135, 135).ToColor;

        public static Rect Square(this Rect r)
        {
            return new Rect(r.x, r.y, Math.Min(r.width, r.height), Math.Min(r.width, r.height));
        }

        public static Rect Square(this Rect r, float f)
        {
            return new Rect(r.x, r.y, f, f);
        }

        public static Rect TextArea(this Rect r, float width = -1)
        {
            return new Rect(r.x, r.y, Math.Max(width, r.width), Text.LineHeight);
        }


        public static void IconLabel(Rect rect, Texture2D icon, string text)
        {
            Rect iconRect = rect.Square();
            Widgets.DrawTextureFitted(iconRect, icon, 1f);

            Rect labelRect = rect;
            labelRect.xMin += iconRect.width;
            Widgets.Label(labelRect, text);
        }

        public static float FeatureLabel(Rect rect, StringColor stringColor)
        {
            float length = Text.CalcSize(stringColor.tag ?? stringColor.desc).x;

            Rect solidBox = rect;
            solidBox.width = length + 12f;
            solidBox.height = Text.LineHeight + 4f;
            Widgets.DrawBoxSolid(solidBox, gray);

            Rect label = solidBox.ContractedBy(4f);
            ColorAction( stringColor.color,
                () => TextCentering(()=>Widgets.Label(label, stringColor.tag?? stringColor.desc))
            );

            TooltipDescription(solidBox,stringColor.desc);
            return length;
        }

        public static void BoxTab(Rect rect, List<TabRecord> records) {
            Widgets.DrawMenuSection(rect);

            Rect tab = rect;
            tab.height = 32f;
            tab.width = rect.width / records.Count;
            foreach(TabRecord tr in records)
            {
                Widgets.DrawMenuSection(tab);
                if(tr.Selected)
                {
                    // 選択時にタブ部分の下のとこを連結させるために無理やり上から線を描写して、塗りつぶす。
                    // 本当は微妙で、元のコードみたいに画像用意が一番スマートなんだけど
                    // 色々とできるのでとりあえずこれで。
                    GUI.color = menuColor;
                    Widgets.DrawLineHorizontal(tab.xMin+1f, tab.yMax-1f, tab.width-2f);
                    GUI.color = Color.white;
                }

                TextCentering(tab, tr.label);
                Widgets.DrawHighlightIfMouseover(tab);
                if (Widgets.ButtonInvisible(tab)) tr.clickedAction.Invoke();
                tab.x += tab.width;
            }
        }

        public static void ColorAction(Color c, Action act)
        {
            Color tmp = GUI.color;
            GUI.color = c;
            act.Invoke();
            GUI.color = tmp;
        }

        public static void TextCentering(Action act)
        {
            TextAnchor anchor = Text.Anchor;
            Text.Anchor = TextAnchor.MiddleCenter;
            act.Invoke();
            Text.Anchor = anchor;
        }

        public static void TextCentering(Rect rect, string label)
        {
            TextCentering(()=>Widgets.Label(rect,label));
        }

        public static void TooltipDescription(Rect rect, string str, bool high = false)
        {
            if (Mouse.IsOver(rect))
            {
                if (high) Widgets.DrawHighlight(rect);
                TooltipHandler.TipRegion(rect, () => str, (int)rect.x ^ 73 );
            }
        }

    }
}
